package com.gic.finance.web.controller;

import com.gic.finance.web.vo.*;
import com.gic.open.api.dto.OrderDTO;
import com.gic.open.api.dto.OrderItemDTO;
import com.gic.open.api.service.OrderApiService;
import org.apache.commons.lang.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import com.gic.api.base.commons.ServiceResponse;
import com.gic.commons.util.EntityUtil;
import com.gic.commons.webapi.reponse.RestResponse;
import com.gic.enterprise.dto.BillingOrderDTO;
import com.gic.enterprise.dto.BillingRechargeRecordDTO;
import com.gic.enterprise.error.ErrorCode;
import com.gic.enterprise.response.EnterpriseRestResponse;
import com.gic.enterprise.service.BillingOrderApiService;
import com.gic.enterprise.service.BillingRechargeRecordApiService;
import com.gic.enterprise.utils.ResultControllerUtils;
import com.gic.finance.constant.OrderTypeEnum;
import com.gic.finance.dto.TransferAccountsApprovalDTO;
import com.gic.finance.qo.TransferListQueryQO;
import com.gic.finance.service.TransferAccountsApprovalApiService;

import java.util.ArrayList;
import java.util.List;

/**
 * 转账审批
 * @ClassName: TransferAccountsApprovalController 
 * @Description:  
 * @author guojuxing 
 * @date 2019/8/9 5:05 PM 
 */
@RestController
@RequestMapping("/transfer-accounts-approval")
public class TransferAccountsApprovalController {
    private static final Logger LOGGER = LoggerFactory.getLogger(TransferAccountsApprovalController.class);

    @Autowired
    private TransferAccountsApprovalApiService transferAccountsApprovalApiService;
    @Autowired
    private BillingOrderApiService billingOrderApiService;
    @Autowired
    private BillingRechargeRecordApiService billingRechargeRecordApiService;
    @Autowired
    private OrderApiService orderApiService;

    @RequestMapping("/get-approval")
    public RestResponse getById(Integer id) {
        ServiceResponse<TransferAccountsApprovalDTO> transferResult = transferAccountsApprovalApiService.getById(id);
        if (transferResult.isSuccess()) {
            TransferAccountsApprovalDTO transfer = transferResult.getResult();
            //短信套餐包
            if (transfer.getOrderType().intValue() == OrderTypeEnum.SHORT_MESSAGE_PACKAGE.getCode()) {
                ServiceResponse<BillingOrderDTO> orderDetail = billingOrderApiService
                        .getOrderBySerialNumber(transfer.getOrderNumber());

                TransferApprovalSmsVO vo = EntityUtil.changeEntityNew(TransferApprovalSmsVO.class, transfer);
                if (orderDetail.isSuccess()) {
                    BillingOrderDTO order = orderDetail.getResult();
                    //订单详情
                    vo.setOrderItemJson(order.getItemJson());
                    vo.setPayInfoSerialNumber(order.getPayInfoSerialNumber());
                    vo.setCouponCardContent(order.getCouponCardContent());
                    vo.setCouponCardName(order.getCouponCardName());
                    vo.setTotalFeePaid(order.getTotalFeePaid());
                    vo.setTotalFee(order.getTotalFee());
                    //总计费用：计划金额 - 优惠券
                    if (order.getCouponCardContent() == null) {
                        order.setCouponCardContent(0D);
                    }
                    vo.setTotalCountFee(transfer.getPlannedAmount() - order.getCouponCardContent());
                    return RestResponse.success(vo);
                } else {
                    return EnterpriseRestResponse.failure(orderDetail);
                }
            } else if (transfer.getOrderType().intValue() == OrderTypeEnum.RECHARGE.getCode()) {
                TransferApprovalRechargeVO vo = EntityUtil.changeEntityNew(TransferApprovalRechargeVO.class, transfer);

                ServiceResponse<BillingRechargeRecordDTO> rechargeResult = billingRechargeRecordApiService
                        .getRechargeRecordForApproval(transfer.getOrderNumber());
                if (rechargeResult.isSuccess()) {
                    BillingRechargeRecordDTO recharge = rechargeResult.getResult();
                    //支付流水号
                    vo.setPayInfoSerialNumber(recharge.getPayInfoSerialNumber());
                    vo.setTotalCountFee(transfer.getPlannedAmount());
                    return RestResponse.success(vo);
                } else {
                    return EnterpriseRestResponse.failure(rechargeResult);
                }
            } else {
                TransferApprovalServiceVO vo = EntityUtil.changeEntityNew(TransferApprovalServiceVO.class, transfer);

                ServiceResponse<OrderDTO> orderResult = orderApiService.getOrderForFinance(transfer.getOrderNumber());
                if (orderResult.isSuccess()) {
                    OrderDTO orderDTO = orderResult.getResult();
                    vo.setOrderCreateTime(orderDTO.getCreateTime());
                    //支付类型
                    vo.setPayType(orderDTO.getPayType());
                    //支付流水号
                    vo.setPayInfoSerialNumber(orderDTO.getPayNumber());
                    vo.setTotalCountFee(transfer.getPlannedAmount());
                    vo.setRelationOrderType(orderDTO.getOrderType());
                    vo.setOperType(orderDTO.getOperType());
                    vo.setOperName(orderDTO.getOperName());
                    String couponName = orderDTO.getCouponName();
                    if (!StringUtils.isBlank(couponName)) {
                        ServiceDiscountVO discountVO = new ServiceDiscountVO();
                        discountVO.setDiscountDetail(orderDTO.getDiscountDetail());
                        discountVO.setDiscountName(couponName);
                        vo.setServiceDiscountVO(discountVO);
                    }
                    if (orderDTO.getOrderItemList() != null) {
                        List<ServiceInfoVO> serviceInfoVOList = new ArrayList<>();
                        List<ExpandPackageVO> expandPackageVOList = new ArrayList<>();
                        List<LicenseVO> licenseVOList = new ArrayList<>();
                        for (OrderItemDTO itemDTO : orderDTO.getOrderItemList()) {
                            if (itemDTO.getServeType() == 1) {
                                //服务
                                ServiceInfoVO serviceInfoVO = new ServiceInfoVO();
                                serviceInfoVO.setServeName(itemDTO.getServeName());
                                serviceInfoVO.setSkuInfo(itemDTO.getSkuInfo());
                                serviceInfoVO.setTotalAmount(itemDTO.getTotalAmount());
                                serviceInfoVO.setServeInfo(itemDTO.getServeInfo());
                                serviceInfoVOList.add(serviceInfoVO);
                            } else if (itemDTO.getServeType() == 2) {
                                //license
                                LicenseVO licenseVO = new LicenseVO();
                                licenseVO.setLicenseUnit(itemDTO.getLicenseUnit());
                                licenseVO.setServeName(itemDTO.getServeName());
                                licenseVO.setTotalAmount(itemDTO.getTotalAmount());
                                licenseVO.setServeQuantity(itemDTO.getServeQuantity());
                                licenseVO.setServeInfo(itemDTO.getServeInfo());
                                licenseVOList.add(licenseVO);
                            } else if (itemDTO.getServeType() == 3) {
                                //拓展包
                                ExpandPackageVO expandPackageVO = new ExpandPackageVO();
                                expandPackageVO.setImageUrl(itemDTO.getImageUrl());
                                expandPackageVO.setServeName(itemDTO.getServeName());
                                expandPackageVO.setServeQuantity(itemDTO.getServeQuantity());
                                expandPackageVO.setTotalAmount(itemDTO.getTotalAmount());
                                expandPackageVO.setServeInfo(itemDTO.getServeInfo());
                                expandPackageVOList.add(expandPackageVO);
                            }
                        }
                        if (serviceInfoVOList.size() > 0) {
                            vo.setServiceInfoVOList(serviceInfoVOList);
                        }
                        if (licenseVOList.size() > 0) {
                            vo.setLicenseVOList(licenseVOList);
                        }
                        if (expandPackageVOList.size() > 0) {
                            vo.setExpandPackageVOList(expandPackageVOList);
                        }
                    }
                    return RestResponse.success(vo);
                } else {
                    return EnterpriseRestResponse.failure(orderResult);
                }
            }
        } else {
            return EnterpriseRestResponse.failure(transferResult);
        }
    }

    @RequestMapping("/agree-approval")
    public RestResponse agreeApproval(Integer id, Double accountAmount) {
        return ResultControllerUtils.commonResult(transferAccountsApprovalApiService.agreeApproval(id, accountAmount));
    }

    @RequestMapping("/reject-approval")
    public RestResponse rejectApproval(Integer id) {
        return ResultControllerUtils.commonResult(transferAccountsApprovalApiService.rejectApproval(id));
    }

    @RequestMapping("/cancel-approval")
    public RestResponse cancelApproval(Integer id) {
        return ResultControllerUtils.commonResult(transferAccountsApprovalApiService.cancelApproval(id));
    }

    @RequestMapping("/list-approval")
    public RestResponse listTransferAccountsApproval(TransferListQueryQO params) {
        return ResultControllerUtils.commonPageResult(
                transferAccountsApprovalApiService.listTransferAccountsApproval(params), TransferApprovalListVO.class);
    }
}
